/**
 * DetachController.java
 *
 * Implements a Controller for the Detacher class.
 *
 * @author Dj
 */

/******************************************************************************

Copyright (c) 2002, D.J.Walker-Morgan/RunState

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

Neither the name of the RunState nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

******************************************************************************/


package com.runstate.detach;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.beans.*;
import java.io.*;
import java.util.*;
import java.net.*;
import javax.mail.*;

import com.runstate.detach.properties.*;
import com.runstate.detach.utils.*;


public class DetachController extends JFrame implements PropertyChangeListener
{

	// Version string for about box
	static final String abouttext="Detach - A Mail Attachment stripper\n(C) Dj@runstate.com\nIcons mostly (C) Dean Gallant\nVersion 0.16 140202-1000";
	
	Detacher d;
	
	DetachPropertiesList dpl=new DetachPropertiesList();
	DetachPropertiesListModel dplm=new DetachPropertiesListModel(dpl);
	DetachProperties currselection=null;
	boolean changed=false;
	
	CardLayout cards=new CardLayout();
	JPanel editorview=new JPanel(cards);
	DetachPropertiesCustomizer dpe=new DetachPropertiesCustomizer();
	JPanel blank=new JPanel();
	
	JList configs=new JList(dplm);
	JButton newbutton=new JButton("New",IconUtils.getImageIcon("com/runstate/detach/icons/New.gif","New"));
	JButton copybutton=new JButton("Copy",IconUtils.getImageIcon("com/runstate/detach/icons/Copy.gif","Copy"));
	JButton delbutton=new JButton("Delete",IconUtils.getImageIcon("com/runstate/detach/icons/Delete.gif","Delete"));
	JButton scanbutton=new JButton("Scan",IconUtils.getImageIcon("com/runstate/detach/icons/ComputerIn.gif","Scan"));
	JButton savebutton=new JButton("Save",IconUtils.getImageIcon("com/runstate/detach/icons/Save.gif","Save"));
	JProgressBar msgprogressbar=new JProgressBar();
	JProgressBar attprogressbar=new JProgressBar();
	JTextField   statustext=new JTextField();
	
	JMenuItem openMenuItem = new JMenuItem("Open");
	JMenuItem saveMenuItem = new JMenuItem("Save");
	JMenuItem saveAsMenuItem = new JMenuItem("Save As");
	JMenuItem exitMenuItem = new JMenuItem("Exit");
	
	boolean scanrunning=false;
	
	JToolBar actions=new JToolBar();
	
	String currentfilename="run.dmc";
	
	public DetachController()
	{
		super("Detach");
		
		//setSize(512,256);
		
		// Add menu.
		
		ActionListener exitMenuItemListener = new ActionListener()
		{
			/**
			 * Time to go!
			 */
			public void actionPerformed(ActionEvent e)
			{
				DetachController.this.exitdialog();
			}
		};
		
		ActionListener openMenuItemListener = new ActionListener()
		{
			/**
			 * Time to go!
			 */
			public void actionPerformed(ActionEvent evt)
			{
				JFileChooser jfc=new JFileChooser(currentfilename);
				jfc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
				int result=jfc.showDialog (null, null);
				if(result!=JFileChooser.APPROVE_OPTION)
				{
					return;
				}
				else
				{
					try
					{
						DetachController.this.openConfig(jfc.getSelectedFile().getAbsolutePath());
					}
					catch (FileNotFoundException e) {}
				}
			}
		};
		
		ActionListener saveMenuItemListener = new ActionListener()
		{
			/**
			 * Time to go!
			 */
			public void actionPerformed(ActionEvent evt)
			{
				
				try
				{
					DetachController.this.saveConfig(currentfilename);
				}
				catch (FileNotFoundException e) {}
				
			}
		};
		
		ActionListener saveAsMenuItemListener = new ActionListener()
		{
			/**
			 * Time to go!
			 */
			public void actionPerformed(ActionEvent evt)
			{
				JFileChooser jfc=new JFileChooser(currentfilename);
				jfc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
				int result=jfc.showDialog (null, null);
				if(result!=JFileChooser.APPROVE_OPTION)
				{
					return;
				}
				else
				{
					try
					{
						String filename=jfc.getSelectedFile().getAbsolutePath();
						DetachController.this.saveConfig(filename);
					}
					catch (FileNotFoundException e) {}
				}
				
			}
		};
	
		ActionListener aboutMenuItemListener = new ActionListener()
		{
			public void actionPerformed(ActionEvent evt)
			{
				JOptionPane.showMessageDialog(DetachController.this,
											  abouttext,
											  "About..",
											  JOptionPane.PLAIN_MESSAGE,
											  IconUtils.getImageIcon("com/runstate/detach/icons/about.gif","About"));
			}
		};
		
		ActionListener scanButtonListener = new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				DetachController.this.scan(currselection);
			}
		};
		
		ActionListener newButtonListener = new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				String newname=JOptionPane.showInputDialog("Enter name");
				if(newname==null) return;
				DetachProperties dp=new DetachProperties();
				dp.setName(newname);
				dpl.add(dp);
				currselection=dp;
				configs.setSelectedValue(dp,true);
				setButtonState();
			}
		};
		
		ActionListener copyButtonListener = new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				try
				{
					DetachProperties dp=(DetachProperties)ObjectCloner.deepCopy(currselection);
					dp.setName("Copy of "+dp.getName());
					dpl.add(dp);
					currselection=dp;
					configs.setSelectedValue(dp,true);
					setButtonState();
				}
				catch (Exception ce)
				{
				}
			}
		};
		
		ActionListener delButtonListener = new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				if(currselection==null)
				{
					return;
				}
				String name=currselection.getName();
				int sure=JOptionPane.showConfirmDialog(DetachController.this,"Are you sure you want to delete '"+name+"'?","Delete?",JOptionPane.YES_NO_OPTION);
				if(sure==JOptionPane.YES_OPTION)
				{
					dpl.remove(currselection);
					configs.clearSelection();
					currselection=null;
					setButtonState();
				}
			}
		};
		
		ListSelectionListener listListener = new ListSelectionListener()
		{
			/**
			 ** Time to get busy!
			 **/
			
			public void valueChanged(ListSelectionEvent e)
			{
				JList j=(JList)e.getSource();
				DetachProperties d=(DetachProperties)j.getSelectedValue();
				if(d==null)
				{
					cards.show(editorview,"blank");
					currselection=null;
					setButtonState();
				}
				else
				{
					cards.show(editorview,"editor");
					dpe.setObject(d);
					currselection=d;
					setButtonState();
				}
			}
			
		};
		
		configs.addListSelectionListener(listListener);
		configs.setFixedCellWidth(128);
		
		openMenuItem.addActionListener(openMenuItemListener);
		saveMenuItem.addActionListener(saveMenuItemListener);
		saveAsMenuItem.addActionListener(saveAsMenuItemListener);
		exitMenuItem.addActionListener(exitMenuItemListener);
		
		JMenu fileMenu = new JMenu("File");
		
		fileMenu.add(openMenuItem);
		fileMenu.add(saveMenuItem);
		fileMenu.add(saveAsMenuItem);
		fileMenu.add(exitMenuItem);
		
		JMenuItem aboutMenuItem=new JMenuItem("About");
		aboutMenuItem.addActionListener(aboutMenuItemListener);
		JMenu helpMenu=new JMenu("Help");
		helpMenu.add(aboutMenuItem);
		
		JMenuBar menuBar = new JMenuBar();
		menuBar.add(fileMenu);
		menuBar.add(Box.createHorizontalGlue());
		menuBar.add(helpMenu);
		
		setJMenuBar(menuBar);
		
		scanbutton.addActionListener(scanButtonListener);
		newbutton.addActionListener(newButtonListener);
		copybutton.addActionListener(copyButtonListener);
		delbutton.addActionListener(delButtonListener);
		savebutton.addActionListener(saveMenuItemListener);
		
		
		// Add window listener.
		
		this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		
		this.addWindowListener
		(
		 new WindowAdapter()
		 {
				/**
				 * Called when window close button was pressed.
				 */
				public void windowClosing(WindowEvent e)
				{
					DetachController.this.exitdialog();
				}
			}
		);
		
		
		
		// TODO: Add your components...
		
		getContentPane().setLayout(new BorderLayout());
		
		actions.setFloatable(false);
		
		Container p=getContentPane();
		
		editorview.add("editor",dpe);
		editorview.add("blank",blank);
		
		JPanel configpanel=new JPanel(new GridBagLayout());
		addGB(configpanel,new JScrollPane(configs),0,0);
		addGB(configpanel,editorview,1,0);
		
		p.add(configpanel,BorderLayout.CENTER);
		
		configpanel.setBorder(new EtchedBorder(EtchedBorder.RAISED));
		editorview.setBorder(new EtchedBorder(EtchedBorder.LOWERED));
		cards.show(editorview,"blank");
			
		actions.add(newbutton);
		actions.add(copybutton);
		actions.add(delbutton);
		actions.add(scanbutton);
		actions.add(savebutton);
		
		p.add(actions,BorderLayout.NORTH);
				
		JPanel statuspanel=new JPanel(new GridBagLayout());
		
		msgprogressbar.setStringPainted(true);
		msgprogressbar.setString("Idle");
		attprogressbar.setStringPainted(true);
		attprogressbar.setString("");
		addGB(statuspanel,msgprogressbar,0,0);
		addGB(statuspanel,attprogressbar,0,1);
		addGB(statuspanel,statustext,0,2);
		statustext.setEditable(false);
		
		p.add(statuspanel,BorderLayout.SOUTH);
		
		statuspanel.setBorder(new EtchedBorder(EtchedBorder.RAISED));
		
		pack();
		
		try
		{
			openConfig(currentfilename);
		}
		catch (FileNotFoundException e)
		{
			dplm.set(dpl);
			dpl.addPropertyChangeListener(this);
			changed=true;
			this.setTitle("Detach - "+currentfilename);
			setButtonState();
		}
	}
	
	GridBagConstraints constraints=new GridBagConstraints();
	
	private void addGB(Container jp,Component component,int x,int y)
	{
		constraints.weightx=1.0;
		constraints.weighty=1.0;
		constraints.insets=new Insets(3,3,3,3);
		constraints.fill=GridBagConstraints.BOTH;
		constraints.gridx=x;
		constraints.gridy=y;
		jp.add(component,constraints);
	}
	
	public void show()
	{
		// You can also use: Dimension size = new Dimension(300,200);
		Dimension size = getPreferredSize();
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		setBounds
		(
			 (screenSize.width - size.width) / 2,
			 (screenSize.height - size.height) / 2,
		 size.width,
		 size.height
		);
		
		super.show();
	}
	
	public void exitdialog()
	{
		String msg="";
		if(changed)
		{
			msg=msg+"You have made changes you haven't saved!\n";
		}
		if(scanrunning)
		{
			msg=msg+"There is a scan in progress!\n";
		}
		if(!msg.equals(""))
		{
			msg=msg+"Are you sure you want to exit?";
		
			int sure=JOptionPane.showConfirmDialog(DetachController.this,msg,"Exit?",JOptionPane.YES_NO_OPTION,JOptionPane.WARNING_MESSAGE);
			if(sure==JOptionPane.YES_OPTION)
			{
				DetachController.this.close();
			}
		}
		else
		{
			DetachController.this.close();
		}
	}
	
	public void openConfig(String filename) throws FileNotFoundException
	{
		XMLDecoder x=new XMLDecoder(new BufferedInputStream(new FileInputStream(filename)));
		Object o=x.readObject();
		configs.clearSelection();
		cards.show(editorview,"blank");
		dpl=(DetachPropertiesList)o;
		dplm.set(dpl);
		dpl.addPropertyChangeListener(this);
		this.setTitle("Detach - "+filename);
		setButtonState();
	}

	public void saveConfig(String filename) throws FileNotFoundException
	{
		XMLEncoder x=new XMLEncoder(new BufferedOutputStream(new FileOutputStream(currentfilename)));
		x.writeObject(dpl);
		x.close();
		changed=false;
		setButtonState();
	}
					
	public void propertyChange(PropertyChangeEvent evt)
	{
		changed=true;
		setButtonState();
	}
	

	/**
	 * Closes the frame.
	 */
	private void close()
	{
		setVisible(false);
		
		System.exit(0); // Delete this line if necessary.
	}
	
	private void setButtonState()
	{
		
		
		if(currselection==null)
		{
			copybutton.setEnabled(false);
			delbutton.setEnabled(false);
			scanbutton.setEnabled(false);
		}
		else
		{
			copybutton.setEnabled(true);
			delbutton.setEnabled(true);
			scanbutton.setEnabled(!scanrunning);
		}
		
		savebutton.setEnabled(changed);
		saveMenuItem.setEnabled(changed);
	}
		
	private void scan(DetachProperties dp)
	{
		if(dp==null)
		{
			return;
		}
		
		if(scanrunning)
		{
			// Should not be here
			return;
		}
		
		final DetachProperties tdp=dp;
		
		final Detacher d=new Detacher();
		
		Runnable scanner=new Runnable(){
			public void run()
			{
				String result=null;
				
	//			try
	//			{
					
					result=d.scan(DetachController.this,tdp);
					
					if(result!=null)
					{
						JOptionPane.showMessageDialog(DetachController.this,result,"Error!",JOptionPane.ERROR_MESSAGE);
						DetachController.this.doneScanning("Last Error :"+result);
					}
	//			}
	//			catch (Exception e)
	//			{
	//				e.printStackTrace();
	//				DetachController.this.doneScanning("Error during scan");
	//			}
			}
		};
		
		Thread t=new Thread(scanner);
		t.start();
		
		scanrunning=true;
		setButtonState();
	}
	
	public void setMsgStatus(int curr,int min,int max,String s)
	{
		msgprogressbar.setMinimum(min);
		msgprogressbar.setMaximum(max-1);
		msgprogressbar.setValue(curr);
		msgprogressbar.setString("Message "+(curr+1)+" of "+max);
		statustext.setText(s);
	}
	
	public void setAttStatus(int curr,int min,int max)
	{
		attprogressbar.setMinimum(min);
		attprogressbar.setMaximum(max-1);
		attprogressbar.setValue(curr);
		attprogressbar.setString("Part "+(curr+1)+" of "+max);
	}
	
	public void clearAttStatus()
	{
		attprogressbar.setMinimum(0);
		attprogressbar.setMaximum(0);
		attprogressbar.setValue(0);
		attprogressbar.setString("");
	}
	
	public void clearMsgStatus()
	{
		msgprogressbar.setMinimum(0);
		msgprogressbar.setMaximum(0);
		msgprogressbar.setValue(0);
		msgprogressbar.setString("Idle");
		statustext.setText("");
	}
	
	public void updateStatusText(String s)
	{
		statustext.setText(s);
	}
	
	public void doneScanning(String result)
	{
		clearMsgStatus();
		clearAttStatus();
		scanrunning=false;
		setButtonState();
		statustext.setText(result);
	}
	
	

}

