/**
 * DetachProperties.java
 *
 * @author Dj
 */

/******************************************************************************

Copyright (c) 2002, D.J.Walker-Morgan/RunState

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

Neither the name of the RunState nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

******************************************************************************/

package com.runstate.detach.properties;

import java.io.*;
import java.beans.*;

public class DetachProperties implements Serializable
{
	String name;
	
	public static int DELETE_NEVER=0;
	public static int DELETE_SAVED_ATTACH=1;
	public static int DELETE_ALL_ATTACH=2;
	public static int DELETE_ALL=3;
	
	String server;
	String proto="imap";
	String user;
	String password;
	String folder;
	String localdir;
	boolean skipDuplicates=false;
	boolean smartPath=false;
	
	int deletemode=DELETE_NEVER;
	
	String  filter="";

	
	private PropertyChangeSupport changes = new PropertyChangeSupport(this);

	public DetachProperties()
	{
	}
	
	public void setName(String name)
	{
		String oldname=this.name;
		this.name = name;
		changes.firePropertyChange("name",oldname,name);
	}
	
	public String getName()
	{
		return name;
	}
	
	public void setServer(String server)
	{
		String oldserver=this.server;
		this.server = server;
		changes.firePropertyChange("server",oldserver,server);
	}
	
	public String getServer()
	{
		return server;
	}
	
	public void setProto(String proto)
	{
		String oldproto=this.proto;
		this.proto = proto;
		changes.firePropertyChange("proto",oldproto,proto);
	}
	
	public String getProto()
	{
		return proto;
	}
	
	public void setUser(String user)
	{
		String olduser=this.user;
		this.user = user;
		changes.firePropertyChange("user",olduser,user);
	}
	
	public String getUser()
	{
		return user;
	}
	
	public void setPassword(String password)
	{
		String oldpassword=this.password;
		this.password = password;
		changes.firePropertyChange("password",oldpassword,password);
	}
	
	public String getPassword()
	{
		return password;
	}
	
	public void setFolder(String folder)
	{
		if(proto.equals("pop3")) return;
		
		String oldfolder=this.folder;
		this.folder = folder;
		changes.firePropertyChange("folder",oldfolder,folder);
	}
	
	public String getFolder()
	{
		if(proto.equals("imap")) return folder;
		if(proto.equals("pop3")) return "INBOX";
		return "";
	}
	
	public void setLocaldir(String localdir)
	{
		String oldlocaldir=this.localdir;
		this.localdir = localdir;
		changes.firePropertyChange("localdir",oldlocaldir,localdir);
	}
	
	public String getLocaldir()
	{
		return localdir;
	}
	
	public void setDeleteMode(int deletemode)
	{
		int olddeletemode=this.deletemode;
		this.deletemode=deletemode;
		changes.firePropertyChange("deleteMode",olddeletemode,deletemode);
	}
	
	public int getDeleteMode()
	{
		return deletemode;
	}
	
	public void setSmartPath(boolean smartPath)
	{
		boolean oldsmartpath=this.smartPath;
		this.smartPath = smartPath;
		changes.firePropertyChange("smartPath",oldsmartpath,smartPath);
	}
	
	public boolean isSmartPath()
	{
		return smartPath;
	}
	
	public void setSkipDuplicates(boolean skipDuplicates)
	{
		boolean oldskipduplicates=this.skipDuplicates;
		this.skipDuplicates = skipDuplicates;
		changes.firePropertyChange("skipDuplicate",oldskipduplicates,skipDuplicates);
	}
	
	public boolean isSkipDuplicates()
	{
		return skipDuplicates;
	}
	
	public void setFilter(String filter)
	{
		String oldfilter=this.filter;
		this.filter = filter;
		changes.firePropertyChange("Filter",oldfilter,filter);
	}
	
	public String getFilter()
	{
		return filter;
	}
	
	public void addPropertyChangeListener(PropertyChangeListener l)
	{
    	changes.addPropertyChangeListener(l);
	}

	public void removePropertyChangeListener(PropertyChangeListener l)
	{
    	changes.removePropertyChangeListener(l);
	}

	public String toString()
	{
		return this.name;
	}
}

