/**
 * IMAPFolderSelectorDialog.java
 *
 * @author Dj
 */

/******************************************************************************

Copyright (c) 2002, D.J.Walker-Morgan/RunState

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

Neither the name of the RunState nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

******************************************************************************/

package com.runstate.detach.properties;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import javax.mail.*;
import javax.mail.internet.*;

public class IMAPFolderSelectorDialog extends JDialog
{

	JTree folderlist;
	JButton okbutton=new JButton("Ok");
	JButton cancelbutton=new JButton("Cancel");
	Session session;
	Store store;
	Folder folder;
	FolderNode top;
	FolderNode currselect=null;
	
	public IMAPFolderSelectorDialog(String title,String host,String username,String password)
	{
		super((JFrame)null,title,true);
		
		try
		{
			buildTree(host,username,password);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		
		folderlist=new JTree(top);
		
		Container p=this.getContentPane();
		p.setLayout(new BorderLayout());
		
		p.add(new JScrollPane(folderlist),BorderLayout.CENTER);
		
		ActionListener cancelbuttonaction=new ActionListener() {
			public void actionPerformed(ActionEvent e)
			{
				currselect=null;
				IMAPFolderSelectorDialog.this.setVisible(false);
			}
		};
		
		ActionListener okbuttonaction=new ActionListener() {
			public void actionPerformed(ActionEvent e)
			{
				IMAPFolderSelectorDialog.this.setVisible(false);
			}
		};
		
		TreeSelectionListener listselection=new TreeSelectionListener()
		{
			public void valueChanged(TreeSelectionEvent e)
			{
				//System.out.println(tse);
				Object[] po=e.getPath().getPath();
				Object o=po[po.length-1];
				if(o instanceof FolderNode)
				{
					currselect=(FolderNode)o;
				}
		
				
			}
		};
		
		okbutton.addActionListener(okbuttonaction);
		cancelbutton.addActionListener(cancelbuttonaction);
		
		folderlist.addTreeSelectionListener(listselection);
		DefaultTreeSelectionModel dtsm=new DefaultTreeSelectionModel();
		dtsm.setSelectionMode(DefaultTreeSelectionModel.SINGLE_TREE_SELECTION);
		folderlist.setSelectionModel(dtsm);
		JPanel buttons=new JPanel();
		buttons.add(okbutton);
		buttons.add(cancelbutton);
		
		p.add(buttons, BorderLayout.SOUTH);
		pack();
		
	}
	
	public String getSelectedFolder()
	{
		if(currselect==null) return null;
		
		return currselect.getFullName();
	}
	
	public void buildTree(String host,String username,String password) throws Exception
	{
		session=Session.getDefaultInstance(System.getProperties(),null);
		session.setDebug(false);
		
		store=session.getStore("imap");
		store.connect(host,username,password);
		
		folder=store.getDefaultFolder();
		
		top=new FolderNode(folder);
		top.setDisplayName(host+"/"+username);
		traverse(top,top);
	}
	
	void traverse(FolderNode t,FolderNode f) throws Exception
	{
		if((f.getType() & Folder.HOLDS_FOLDERS) != 0)
		{
			Folder[] cf=f.list();
			for(int n=0;n<cf.length;n++)
			{
				FolderNode nf=new FolderNode(cf[n]);
				f.add(nf);
				traverse(f,nf);
			}
		}
	}
			
	public class FolderNode extends DefaultMutableTreeNode
	{
		Folder f;
		String displayname=null;
		
		FolderNode(Folder f)
		{
			this.f=f;
		}
		
		public String toString()
		{
			if(displayname==null)
			{
				return f.getName();
			}
			
			return displayname;
		}
		
		public String getFullName()
		{
			return f.getFullName();
		}
		
		public int getType() throws MessagingException
		{
			return f.getType();
		}
		
		public void setDisplayName(String s)
		{
			displayname=s;
		}
		
		public Folder[] list() throws MessagingException
		{
			return f.list();
		}
	}
	
}

